/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::ObjectPoseClientExample
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "ObjectPoseClientExample.h"

#include <ArmarXCore/core/time/CycleUtil.h>


namespace armarx
{

    armarx::PropertyDefinitionsPtr ObjectPoseClientExample::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs = new ComponentPropertyDefinitions(getConfigIdentifier());

        return defs;
    }

    std::string ObjectPoseClientExample::getDefaultName() const
    {
        return "ObjectPoseClientExample";
    }

    void ObjectPoseClientExample::onInitComponent()
    {
    }

    void ObjectPoseClientExample::onConnectComponent()
    {
        setDebugObserverBatchModeEnabled(true);

        objectProcessingTask = new SimpleRunningTask<>([this]()
        {
            this->objectProcessingTaskRun();
        });
        objectProcessingTask->start();
    }

    void ObjectPoseClientExample::onDisconnectComponent()
    {
    }

    void ObjectPoseClientExample::onExitComponent()
    {
    }


    void ObjectPoseClientExample::objectProcessingTaskRun()
    {
        CycleUtil cycle(50);

        while (objectProcessingTask && !objectProcessingTask->isStopped())
        {
            const objpose::ObjectPoseSeq objectPoses = ObjectPoseClient::getObjectPoses();

            ARMARX_VERBOSE << "Received poses of " << objectPoses.size() << " objects.";

            {
                setDebugObserverDatafield("NumObjectPoses", objectPoses.size());
                sendDebugObserverBatch();
            }

            {
                // Visualize the objects.
                viz::Layer layer = arviz.layer("Objects");
                for (const objpose::ObjectPose& objectPose : objectPoses)
                {
                    layer.add(viz::Object(objectPose.objectID.str())
                              .pose(objectPose.objectPoseGlobal)
                              .fileByObjectFinder(objectPose.objectID)
                              .alpha(objectPose.confidence));
                }
                arviz.commit(layer);
            }

            cycle.waitForCycleDuration();
        }
    }
}
