#pragma once

#include <mutex>

#include <ArmarXCore/core/ComponentPlugin.h>

#include <RobotAPI/interface/armem/server/MemoryInterface.h>
#include <RobotAPI/interface/armem/client/MemoryListenerInterface.h>
#include <RobotAPI/interface/armem/mns/MemoryNameSystemInterface.h>

#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>
#include <RobotAPI/libraries/armem/server/ltm/mongodb/MemoryManager.h>
#include <RobotAPI/libraries/armem/client/MemoryNameSystemComponentPlugin.h>

#include "MemoryToIceAdapter.h"


namespace armarx::armem::server
{
    class ComponentPluginUser;
}


namespace armarx::armem::server::plugins
{

    class ComponentPlugin : public client::plugins::MemoryNameSystemComponentPlugin
    {
        using Base = client::plugins::MemoryNameSystemComponentPlugin;
    public:

        using Base::MemoryNameSystemComponentPlugin;
        virtual ~ComponentPlugin() override;

        void postCreatePropertyDefinitions(PropertyDefinitionsPtr& properties) override;

        void postOnConnectComponent() override;
        void preOnDisconnectComponent() override;


        /**
         * @brief Register the parent component in the MNS.
         * Called before onConnect() if MNS is enabled.
         */
        data::RegisterMemoryResult registerMemory(ComponentPluginUser& parent);
        /**
         * @brief Remove the parent component from the MNS.
         * Called before onDisconnect() if MNS is enabled.
         */
        data::RemoveMemoryResult removeMemory(ComponentPluginUser& parent);

        client::MemoryListenerInterfacePrx memoryListener;

        std::string longTermMemoryDatabaseHost;
        std::string longTermMemoryDatabaseUser;
        std::string longTermMemoryDatabasePassword;
    };

}


namespace armarx::armem::server
{

    /**
     * @brief Base class of memory server components.
     */
    class ComponentPluginUser :
        virtual public ManagedIceObject
        , virtual public MemoryInterface
        , virtual public client::MemoryNameSystemComponentPluginUser
    {
    public:

        ComponentPluginUser();
        virtual ~ComponentPluginUser() override;

        /// Set the name of the wm and the ltm (if enabled)
        void setMemoryName(const std::string&);


        // WritingInterface interface
        virtual data::AddSegmentsResult addSegments(const data::AddSegmentsInput& input, const Ice::Current& = Ice::emptyCurrent) override;
        data::AddSegmentsResult addSegments(const data::AddSegmentsInput& input, bool addCoreSegments);

        virtual data::CommitResult commit(const data::Commit& commit, const Ice::Current& = Ice::emptyCurrent) override;


        // ReadingInterface interface
        virtual armem::query::data::Result query(const armem::query::data::Input& input, const Ice::Current& = Ice::emptyCurrent) override;


        // StoringInterface interface
        virtual data::StoreResult store(const data::StoreInput&, const Ice::Current& = Ice::emptyCurrent) override;


        // LoadingInterface interface


    public:

        /// The actual memory.
        server::wm::Memory workingMemory;
        // [[deprecated ("The global working memory mutex is deprecated. Use the core segment mutexes instead.")]]
        // std::mutex workingMemoryMutex;

        /// Parameter to indicate whether to use or not to use the ltm feature
        bool longtermMemoryEnabled = true;

        /// A manager class for the ltm. It internally holds a normal wm instance
        server::ltm::mongodb::MemoryManager longtermMemoryManager;

        /// property defaults
        std::string memoryListenerDefaultName = "MemoryUpdates";

        /// Helps connecting `memory` to ice. Used to handle Ice callbacks.
        MemoryToIceAdapter iceMemory { &workingMemory, &longtermMemoryManager};


    private:

        plugins::ComponentPlugin* plugin = nullptr;

    };


}
