#include "TypedDataDisplayVisitor.h"

#include <iomanip>      // std::setprecision

#include <SimoxUtility/algorithm/string.h>

#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/libraries/aron/converter/eigen/EigenConverter.h>
#include <RobotAPI/libraries/armem/core/Time.h>

#include "DataDisplayVisitor.h"


namespace armarx::aron
{

    std::string TypedDataDisplayVisitor::getValue(TypeNavigator& type, DataNavigator& data)
    {
        TypedDataDisplayVisitor v;
        bool r = v.applyTo(type, data);
        ARMARX_CHECK(!r);
        return v.value.str();
    }

    bool TypedDataDisplayVisitor::visitEnter(DictTypeNavigator&, DictDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visitEnter(ObjectTypeNavigator&, DictDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visitEnter(ListTypeNavigator&, ListDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visitEnter(TupleTypeNavigator&, ListDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(BoolTypeNavigator&, BoolDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(DoubleTypeNavigator&, DoubleDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(FloatTypeNavigator&, FloatDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(IntTypeNavigator&, IntDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(LongTypeNavigator&, LongDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(StringTypeNavigator&, StringDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(TimeTypeNavigator&, LongDataNavigator& data)
    {
        armem::Time time = armem::Time::microSeconds(data.getValue());
        armem::toDateTimeMilliSeconds(time);
        value << armem::toDateTimeMilliSeconds(time);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(EigenMatrixTypeNavigator&, NDArrayDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(EigenQuaternionTypeNavigator&, NDArrayDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(IVTCByteImageTypeNavigator&, NDArrayDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(OpenCVMatTypeNavigator&, NDArrayDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(PCLPointCloudTypeNavigator&, NDArrayDataNavigator& data)
    {
        value << DataDisplayVisitor::getValue(data);
        return false;
    }

    bool TypedDataDisplayVisitor::visit(PoseTypeNavigator&, NDArrayDataNavigator& data)
    {
        const Eigen::Matrix4f pose = aron::converter::AronEigenConverter::ConvertToMatrix4f(data);
        value << std::setprecision(2) << std::fixed;
        value << pose.format(Eigen::IOFormat(Eigen::StreamPrecision, 0, coeffSep, "\n", "", "", "", ""));
        return false;
    }

    bool TypedDataDisplayVisitor::visit(PositionTypeNavigator&, NDArrayDataNavigator& data)
    {
        const Eigen::Vector3f pos = aron::converter::AronEigenConverter::ConvertToVector3f(data);
        value << std::setprecision(2) << std::fixed;
        value << pos.format(Eigen::IOFormat(Eigen::StreamPrecision, 0, "", coeffSep, "", "", "", ""));
        return false;
    }

    bool TypedDataDisplayVisitor::visit(OrientationTypeNavigator&, NDArrayDataNavigator& data)
    {
        const Eigen::Quaternionf quat = aron::converter::AronEigenConverter::ConvertToQuaternionf(data);
        value << std::setprecision(2) << std::fixed;
        value << quat.w() << coeffSep << "|" << coeffSep << quat.x() << coeffSep << quat.y() << coeffSep << quat.z();
        return false;
    }

}
