#include "DataTreeBuilder.h"

#include <QTreeWidgetItem>

#include <RobotAPI/libraries/armem_gui/TreeWidgetBuilder.h>


namespace armarx::armem::gui::instance
{

    DataTreeBuilder::DataTreeBuilder()
    {
    }

    void DataTreeBuilder::updateTree(QTreeWidgetItem* parent, const aron::datanavigator::DictNavigatorPtr& data)
    {
        DictBuilder builder = getDictBuilder();
        builder.setUpdateItemFn([this, &data](const std::string & key, QTreeWidgetItem * item)
        {
            auto child = data->getElement(key);
            this->update(item, key, child);
            return true;
        });

        builder.updateTreeWithContainer(parent, data->getAllKeys());
    }

    void DataTreeBuilder::updateTree(QTreeWidgetItem* parent, const aron::datanavigator::ListNavigatorPtr& data)
    {
        auto children = data->getChildren();

        ListBuilder builder = getListBuilder();
        builder.setUpdateItemFn([this, &children](size_t key, QTreeWidgetItem * item)
        {
            this->update(item, std::to_string(key), children.at(key));
            return true;
        });

        builder.updateTreeWithContainer(parent, getIndex(children.size()));
    }


    void DataTreeBuilder::update(QTreeWidgetItem* item, const std::string& key, const aron::datanavigator::NavigatorPtr& data)
    {
        if (!data)
        {
            this->setRowTexts(item, key, "null");
            return;
        }

        this->setRowTexts(item, key, *data);

        if (auto cast = aron::datanavigator::DictNavigator::DynamicCast(data))
        {
            updateTree(item, cast);
        }
        else if (auto cast = aron::datanavigator::ListNavigator::DynamicCast(data))
        {
            updateTree(item, cast);
        }
    }


}
