#pragma once

#include <map>

#include <QWidget>

#include <RobotAPI/libraries/armem/core/DataMode.h>
#include <RobotAPI/libraries/armem/client/query/Builder.h>

#include "SnapshotForm.h"


class QCheckBox;
class QComboBox;
class QVBoxLayout;


namespace armarx::armem::gui
{

    /**
     * @brief Widget where queries for snapshots can be configured.
     *
     * Manages multiple forms which are hidden if not selected
     * (in order to keep their settings).
     */
    class SnapshotSelectorWidget : public QWidget
    {
        Q_OBJECT
        using This = SnapshotSelectorWidget;

    public:

        SnapshotSelectorWidget();


        armem::DataMode dataMode() const;
        client::query::SnapshotSelector selector();
        query::data::QueryTargets queryTargets() const;


    public slots:

    signals:
        void queryChanged();

    private slots:

        //void updateSelector();

        void hideAllForms();
        void showSelectedFormForQuery(QString selected);
        void queryTargetStateChanged();

    signals:
        void queryOutdated();


    private:

        void addForm(const QString& key, SnapshotForm* form);


    public:
        QVBoxLayout* _pageLayout;
        QComboBox* _queryComboBox;
        QCheckBox* _WMQueryTargetCheckBox;
        QCheckBox* _LTMQueryTargetCheckBox;
        /// The forms for the different query types. Hidden when not selected.
        std::map<QString, SnapshotForm*> _queryForms;

    };

}
