/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::armem_gui
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#define BOOST_TEST_MODULE RobotAPI::ArmarXLibraries::armem_gui

#define ARMARX_BOOST_TEST

#include <RobotAPI/Test.h>
// #include "../armem_gui.h"

#include <iostream>

#include <RobotAPI/libraries/armem_gui/instance/sanitize_typename.h>
#include <RobotAPI/libraries/aron/core/navigator/type/AllNavigators.h>

using namespace armarx::armem::gui::instance;


BOOST_AUTO_TEST_CASE(test_sanitizeTypeName)
{
    using namespace armarx::aron::typenavigator;
    {
        DictNavigator dict;
        dict.setAcceptedType(std::make_shared<FloatNavigator>());
        BOOST_CHECK_EQUAL(sanitizeTypeName(dict.getName()), "Dict<Float>");
    }
    {
        ListNavigator list;
        list.setAcceptedType(std::make_shared<LongNavigator>());
        BOOST_CHECK_EQUAL(sanitizeTypeName(list.getName()), "List<Long>");
    }
    {
        ObjectNavigator obj;
        obj.setObjectName("namespace::MyObjectName");
        BOOST_CHECK_EQUAL(sanitizeTypeName(obj.getName()), "MyObjectName    (namespace)");
    }
}
