/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "SerializerFactory.h"

// ArmarX
#include <RobotAPI/libraries/aron/core/codegenerator/codeWriter/cpp/serializer/AllSerializers.h>

namespace armarx::aron::cppserializer
{
    // Access method
    SerializerPtr SerializerFactory::create(const typenavigator::NavigatorPtr& n, const Path& path) const
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
   {type::Descriptor::e##upperType, SerializerFactoryPtr(new upperType##SerializerFactory())},
        static const std::map<type::Descriptor, SerializerFactoryPtr> Factories =
        {
            HANDLE_ALL_ARON_TYPES
        };
#undef RUN_ARON_MACRO

        //CheckIfPtrIsNull("NavigatorFactory", "create", path, n);
        auto factory_iterator = Factories.find(n->getDescriptor());
        if (factory_iterator == Factories.end())
        {
            throw error::AronException("NavigatorFactory", "create", "Cannot find the desired factory for input (ice_id): " + n->toAronPtr()->ice_id() + ". Cannot create navigator", path);
        }
        return factory_iterator->second->createSpecific(n, path);
    }

    SerializerPtr SerializerFactory::createSpecific(const typenavigator::NavigatorPtr&, const Path&) const
    {
        throw error::AronException("NavigatorFactory", "createSpecific", "Called disallowed method. Use child class instead!");
    }

    // Factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    SerializerPtr upperType##SerializerFactory::createSpecific(const typenavigator::NavigatorPtr& n, const Path&) const \
    { \
        typenavigator::upperType##NavigatorPtr casted = typenavigator::upperType##Navigator::DynamicCastAndCheck(n); \
        return SerializerPtr(new serializer::upperType##Serializer(casted)); \
    }

    HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
}
