/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/Config.h>
#include <RobotAPI/libraries/aron/core/Descriptor.h>

namespace armarx::aron::dataIO
{
    class WriterInterface;
    typedef std::shared_ptr<WriterInterface> WriterInterfacePtr;

    class WriterInterface
    {
    public:
        virtual void writeStartDict() = 0;
        virtual void writeEndDict() = 0;
        virtual void writeStartList() = 0;
        virtual void writeEndList() = 0;

        virtual void writeNDArray(const std::vector<int>&, const std::string&, const unsigned char*) = 0;

        virtual void writePrimitive(int) = 0;
        virtual void writePrimitive(long) = 0;
        virtual void writePrimitive(float) = 0;
        virtual void writePrimitive(double) = 0;
        virtual void writePrimitive(const std::string&) = 0;
        virtual void writePrimitive(bool) = 0;

        virtual void writeNull() = 0;

        virtual void writeKey(const std::string&) = 0;
    };
}
