﻿/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/libraries/aron/core/Config.h>


namespace armarx::aron::typeIO
{
    class ReaderInterface;
    typedef std::shared_ptr<ReaderInterface> ReaderInterfacePtr;

    class ReaderInterface
    {
    public:
        struct ReadStartObjectReturnType
        {
            std::string name;
            unsigned int elements;
            type::Maybe maybe;
        };
        virtual ReadStartObjectReturnType readStartObject() = 0;
        virtual bool readEndObject() = 0;

        struct ReadStartListReturnType
        {
            unsigned int elements;
            type::Maybe maybe;
        };
        virtual ReadStartListReturnType readStartList() = 0;
        virtual bool readEndList() = 0;

        struct ReadStartDictReturnType
        {
            unsigned int elements;
            type::Maybe maybe;
        };
        virtual ReadStartDictReturnType readStartDict() = 0;
        virtual bool readEndDict() = 0;

        struct ReadStartTupleReturnType
        {
            unsigned int elements;
            type::Maybe maybe;
        };
        virtual ReadStartTupleReturnType readStartTuple() = 0;
        virtual bool readEndTuple() = 0;

        struct ReadStartPairReturnType
        {
            unsigned int elements;
            type::Maybe maybe;
        };
        virtual ReadStartPairReturnType readStartPair() = 0;
        virtual bool readEndPair() = 0;

        struct ReadEigenMatrixReturnType
        {
            unsigned int rows;
            unsigned int cols;
            std::string type;
            type::Maybe maybe;
        };
        virtual ReadEigenMatrixReturnType readEigenMatrix() = 0;

        struct ReadEigenQuaternionReturnType
        {
            std::string type;
            type::Maybe maybe;
        };
        virtual ReadEigenQuaternionReturnType readEigenQuaternion() = 0;

        struct ReadIVTCbyteImageReturnType
        {
            type::Maybe maybe;
        };
        virtual ReadIVTCbyteImageReturnType readIVTCByteImage() = 0;

        struct ReadOpenCVMatReturnType
        {
            type::Maybe maybe;
        };
        virtual ReadOpenCVMatReturnType readOpenCVMat() = 0;

        struct ReadPCLPointCloudReturnType
        {
            std::string type;
            type::Maybe maybe;
        };
        virtual ReadPCLPointCloudReturnType readPCLPointCloud() = 0;

        struct ReadPositionReturnType
        {
            type::Maybe maybe;
        };
        virtual ReadPositionReturnType readPosition() = 0;

        struct ReadOrientationReturnType
        {
            type::Maybe maybe;
        };
        virtual ReadOrientationReturnType readOrientation() = 0;

        struct ReadPoseReturnType
        {
            type::Maybe maybe;
        };
        virtual ReadPoseReturnType readPose() = 0;

        struct ReadPrimitiveReturnType
        {
            type::Maybe maybe;
        };
        virtual ReadPrimitiveReturnType readInt() = 0;
        virtual ReadPrimitiveReturnType readLong() = 0;
        virtual ReadPrimitiveReturnType readFloat() = 0;
        virtual ReadPrimitiveReturnType readDouble() = 0;
        virtual ReadPrimitiveReturnType readString() = 0;
        virtual ReadPrimitiveReturnType readBool() = 0;
        virtual ReadPrimitiveReturnType readTime() = 0;

        virtual std::string readKey() = 0;
        virtual void loadMember(const std::string&) = 0;

        // Helper functions
        virtual type::Descriptor getTypeOfNext() const = 0;
    };
}
