/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>

// Base class
#include "../detail/NavigatorBase.h"

// ArmarX
#include "Dict.h"
#include "../../type/container/List.h"

namespace armarx::aron::datanavigator
{
    class ListNavigator;
    typedef std::shared_ptr<ListNavigator> ListNavigatorPtr;

    class ListNavigator :
        virtual public detail::NavigatorBase<data::AronList, ListNavigator>
    {
    public:
        // constructors
        ListNavigator(const Path& path = Path());
        ListNavigator(const data::AronListPtr&, const Path& path = Path());
        ListNavigator(const data::AronDataList&, const Path& path = Path());
        ListNavigator(const std::vector<NavigatorPtr>&, const Path& path = Path());

        // operators
        virtual bool operator==(const ListNavigator&) const override;
        bool operator==(const ListNavigatorPtr&) const override;

        // static methods
        static PointerType FromAronListPtr(const data::AronListPtr& aron);
        static data::AronListPtr ToAronListPtr(const PointerType& navigator);

        // public member functions
        DictNavigatorPtr getAsDict() const;

        data::AronListPtr toAronListPtr() const;

        void addElement(const NavigatorPtr&);
        NavigatorPtr getElement(unsigned int) const;
        bool hasElement(unsigned int) const;
        std::vector<NavigatorPtr> getElements() const;

        void clear();

        // virtual implementations
        virtual std::string getName() const override;
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;

        virtual typenavigator::NavigatorPtr recalculateType() const override;
        virtual bool fullfillsType(const typenavigator::NavigatorPtr&) const override;

        virtual NavigatorPtr navigateAbsolute(const Path& path) const override;
        virtual NavigatorPtr navigateRelative(const Path& path) const override;

    private:
        std::vector<NavigatorPtr> childrenNavigators;
    };
}
