/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>

// Base class
#include "../detail/PrimitiveNavigatorBase.h"

// ArmarX
#include "../../type/primitive/Bool.h"

namespace armarx::aron::datanavigator
{

    class BoolNavigator;
    typedef std::shared_ptr<BoolNavigator> BoolNavigatorPtr;

    class BoolNavigator :
        virtual public detail::PrimitiveNavigatorBase<data::AronBool, bool, BoolNavigator>
    {
    public:
        /* constructors */
        BoolNavigator(const Path& = Path());
        BoolNavigator(const data::AronBoolPtr&, const Path& = Path());
        BoolNavigator(const bool, const Path& = Path());

        // operators
        bool operator==(const BoolNavigator& other) const override;
        bool operator==(const BoolNavigatorPtr&) const override;

        // static methods
        static BoolNavigatorPtr FromAronBoolPtr(const data::AronBoolPtr& aron);
        static data::AronBoolPtr ToAronBoolPtr(const BoolNavigatorPtr& navigator);

        // class methods
        data::AronBoolPtr toAronBoolPtr() const;

        /* virtual implementations */
        virtual std::string getName() const override;

        virtual typenavigator::NavigatorPtr recalculateType() const override;
        virtual bool fullfillsType(const typenavigator::NavigatorPtr&) const override;
    };
}
