/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "Int.h"

#include <RobotAPI/libraries/aron/core/navigator/data/NavigatorFactory.h>
#include <RobotAPI/libraries/aron/core/navigator/type/primitive/Int.h>

namespace armarx::aron::datanavigator
{
    /* constructors */
    IntNavigator::IntNavigator(const data::AronIntPtr& o, const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eInt, path),
        detail::NavigatorBase<data::AronInt, IntNavigator>(o)
    {
    }

    IntNavigator::IntNavigator(const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eInt, path)
    {
    }

    IntNavigator::IntNavigator(const int d, const Path& path) :
        IntNavigator(data::AronIntPtr(new data::AronInt(d)), path)
    {
    }

    /* operators */
    bool IntNavigator::operator==(const IntNavigator& other) const
    {
        const auto& otherAron = other.toAronIntPtr();
        if (this->aron->value != otherAron->value)
        {
            return false;
        }
        return true;
    }
    bool IntNavigator::operator==(const IntNavigatorPtr& other) const
    {
        if (!other)
        {
            return false;
        }
        return *this == *other;
    }


    /* static methods */
    IntNavigatorPtr IntNavigator::FromAronIntPtr(const data::AronIntPtr& aron)
    {
        return std::make_shared<IntNavigator>(aron);
    }

    data::AronIntPtr IntNavigator::ToAronIntPtr(const IntNavigatorPtr& navigator)
    {
        return navigator ? navigator->toAronIntPtr() : nullptr;
    }

    /* public member functions */
    data::AronIntPtr IntNavigator::toAronIntPtr() const
    {
        return aron;
    }


    /* virtual implementations */
    std::string IntNavigator::getName() const
    {
        return "data::AronInt";
    }

    bool IntNavigator::fullfillsType(const typenavigator::NavigatorPtr& type) const
    {
        return type->getDescriptor() == type::Descriptor::eInt || type->getDescriptor() == type::Descriptor::eIntEnum;
    }

    typenavigator::NavigatorPtr IntNavigator::recalculateType() const
    {
        return std::make_shared<typenavigator::IntNavigator>(getPath());
    }
}
