/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>

// Base class
#include "../detail/PrimitiveNavigatorBase.h"

// ArmarX
#include "../../type/primitive/Int.h"

namespace armarx::aron::datanavigator
{

    class IntNavigator;
    typedef std::shared_ptr<IntNavigator> IntNavigatorPtr;

    class IntNavigator :
        virtual public detail::PrimitiveNavigatorBase<data::AronInt, int, IntNavigator>
    {
    public:
        /* constructors */
        IntNavigator(const Path& = Path());
        IntNavigator(const data::AronIntPtr&, const Path& = Path());
        IntNavigator(const int, const Path& = Path());

        /* operators */
        bool operator==(const IntNavigator&) const override;
        bool operator==(const IntNavigatorPtr&) const override;

        /* static methods */
        static IntNavigatorPtr FromAronIntPtr(const data::AronIntPtr& aron);
        static data::AronIntPtr ToAronIntPtr(const IntNavigatorPtr& navigator);

        /* public member functions */
        data::AronIntPtr toAronIntPtr() const;

        /* virtual implementations */
        virtual std::string getName() const override;

        virtual typenavigator::NavigatorPtr recalculateType() const override;
        virtual bool fullfillsType(const typenavigator::NavigatorPtr&) const override;
    };
}
