/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "String.h"

namespace armarx::aron::datanavigator
{
    /* constructors */
    StringNavigator::StringNavigator(const data::AronStringPtr& o, const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eString, path),
        detail::NavigatorBase<data::AronString, StringNavigator>(o)
    {
    }

    StringNavigator::StringNavigator(const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eString, path)
    {
    }

    StringNavigator::StringNavigator(const std::string& d, const Path& path) :
        StringNavigator(data::AronStringPtr(new data::AronString(d)), path)
    {
    }

    /* operators */
    bool StringNavigator::operator==(const StringNavigator& other) const
    {
        const auto& otherAron = other.toAronStringPtr();
        if (this->aron->value != otherAron->value)
        {
            return false;
        }
        return true;
    }
    bool StringNavigator::operator==(const StringNavigatorPtr& other) const
    {
        if (!other)
        {
            return false;
        }
        return *this == *other;
    }


    /* static methods */
    StringNavigatorPtr StringNavigator::FromAronStringPtr(const data::AronStringPtr& aron)
    {
        return std::make_shared<StringNavigator>(aron);
    }

    data::AronStringPtr StringNavigator::ToAronStringPtr(const StringNavigatorPtr& navigator)
    {
        return navigator ? navigator->toAronStringPtr() : nullptr;
    }

    /* public member functions */
    data::AronStringPtr StringNavigator::toAronStringPtr() const
    {
        return aron;
    }


    /* virtual implementations */
    std::string StringNavigator::getName() const
    {
        return "data::AronString";
    }

    bool StringNavigator::fullfillsType(const typenavigator::NavigatorPtr& type) const
    {
        return type->getDescriptor() == type::Descriptor::eString;
    }

    typenavigator::NavigatorPtr StringNavigator::recalculateType() const
    {
        return std::make_shared<typenavigator::StringNavigator>(getPath());
    }
}
