/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base class
#include "../detail/NDArrayNavigatorBase.h"

namespace armarx::aron::typenavigator
{
    class OpenCVMatNavigator;
    typedef std::shared_ptr<OpenCVMatNavigator> OpenCVMatNavigatorPtr;

    class OpenCVMatNavigator :
        virtual public detail::NDArrayNavigatorBase<type::AronOpenCVMat, OpenCVMatNavigator>
    {
    public:
        // constructors
        OpenCVMatNavigator(const Path& path = Path());
        OpenCVMatNavigator(const type::AronOpenCVMatPtr&, const Path& path);

        // operators
        virtual bool operator==(const OpenCVMatNavigator&) const override;

        // public member functions
        type::AronOpenCVMatPtr toAronOpenCVMatPtr() const;

        // virtual implementations
        virtual std::string getName() const override;

    public:
        const std::map<std::string, std::vector<std::string>> ACCEPTED_TYPES =
        {
            {"CV_8U", {"8U"}},
            {"CV_8S", {"8S"}},
            {"CV_16U", {"16U"}},
            {"CV_16S", {"16S"}},
            {"CV_32S", {"32S"}},
            {"CV_32F", {"32F"}},
            {"CV_64F", {"64F"}}
        };
    };
}
